classdef mania_class < handle
    % A Pattern-Class which holds Members of a class
    %  and works as a simple interface for all class Members
    %
    properties (SetAccess = public)
        name                    % Human readable name for this class
        testGroup;              % groups, that belong to testing 
    end
    
    properties (SetAccess = protected)
        members;                % cell array of members
        sourceDimension = {};   % original image dimension
    end
    
    methods
        
        function obj = mania_class(className,varargin)
            % constructur with class name & optional the members of the class
            obj.name = className;
            obj.members = {};
            obj.testGroup = false;
            if nargin > 1
                for memberCount=1:size(varargin,2)
                    obj = addMember(obj,varargin{memberCount});
                end
            end
        end
        
        function this = addMember(this,newMember)
            % adds a new member to the class
            if isa(newMember,'mania_pattern')
                this.members{size(this.members,1)+1,1} = newMember;
                if isempty(this.sourceDimension)
                    this.sourceDimension = newMember.sourceDimension;
                elseif ~isequal(newMember.sourceDimension,this.sourceDimension)
                    warning(['Image dimension of Member "' newMember.description '" does not match dimension of other members!']);
                end
            else
                throw(MException('mania:WrongPatternClass','This is not a mania pattern class file'));
            end
        end
        
        function member = getMember(this,memberIndex)
            % returns the member with index "memberIndex"
            if this.size > 0 && this.size >= memberIndex
                member = this.members{memberIndex};
            else
                throw(MException('mania:MemberIndexOutOfBounds','Index does not match class size'));
            end
        end
        
        function members = getAllMembers(this)
            % simply returns all Members
            members = this.members;
        end
        
        function s = size(this)
            % returns amount of Members
            s = size(this.members,1);
        end
        
        function loaded = allLoaded(this)
            % if there are is one object not loaded or no subject at all
            % return false
            if this.size > 0
                loaded = true;
                for mem=1:size(this.members,1)
                    loaded = this.members{mem}.isLoaded;
                end
            else
                loaded = false;
            end
        end
        
        function this = setDimensionalitiyReduction(this,fsmethod)
            % sets for all members the same reduction method, how multiple
            % image data should be handeled.
            try
                for mem=1:size(this.members,1)
                    this.members{mem} = this.members{mem}.setFeatureSelection(fsmethod);
                end
            catch excep
                error('Seems not to be a correct dim. reduction method!')
            end
        end
        
        function tf = reductionMethodsSet(this)
            % returns true, if all reduction methods were set
            tf = true;
            try
                for mem=1:size(this.members,1)
                    this.members{mem}.getFeatureSelection;
                end
            catch
                tf = false;
            end
        end
        
        function this = loadAll(this)
            % Load all Pattern (and reload already loaded patterns)
            for mem=1:size(this.members,1)
                this.members{mem} = this.members{mem}.generatePattern;
            end
        end
        
        function v = voxelsPerSub(this)
        % returns the voxels of all subjects (by looking at the first
        % subject - there is NO validation if alle have the same Voxel
        % amount!!! therefore look at "isValid")
            if this.allLoaded
                v = this.members{1}.getVoxelCount;
            else
                v = 0;
            end
        end
        
        function valid = isValid(this)
            % should return true if everything is ready for computing:
            %   * feature reduction methods should be set
            %   * images should be loaded
            %   * there should be at least ONE subject.
            if this.allLoaded && this.reductionMethodsSet && this.size > 0
                valid = true;
                v = this.voxelsPerSub;
                for mem=1:this.size
                    if this.members{1}.getVoxelCount ~= v
                        valid = false;
                    end
                end
                
            else
                valid = false;
            end
        end
        
        function show(this)
            % prints out a human readable Description of the Class
            fprintf('\nClass:  ::: "%s" :::\n',this.name);
            fprintf('   Members:               %d\n',this.size);
            if this.reductionMethodsSet; r = 'Yes'; else r = 'No';
            end;
            if this.allLoaded; l = 'Yes'; else l = 'No';
            end;
            fprintf('   Reduction Methods set: %s\n', r);
            if l
                fprintf('   Images Loaded:         %s\n', l);
                fprintf('   Voxels per Subject:    %d\n', this.voxelsPerSub);
            end
        end
        
        function showMembers(this)
            % prints out a human readable list of class members
            fprintf('\nMembers of:  ::: "%s" :::\n',this.name);
            fprintf('   Index        Description\n')
            fprintf('   ---------------------------------------\n');
            for mem=1:size(this.members,1)
                fprintf('    %3d.:       %s\n', mem, this.members{mem}.description);
            end
            fprintf('   ---------------------------------------\n');
        end
        
        function markerList = getUniqueMarkers(this)
            % get "Markers", which should/would be unique for a subject (to
            % test for duplicate subjects)
            markerList = cell(1,this.size);
            for mem=1:size(this.members,1)
                markers = struct;
                markerList{mem} = this.members{mem}.getUniqueMarkers(markers);
            end
        end
    end
    
end
