% Abstract class mania_pattern
%   Representation of a (Neuroimage-)Pattern
%
classdef mania_pattern < handle
    
    properties (SetAccess = protected)
        VoxelValues           % The Pattern!
        isLoaded = false;     % true if images were loaded
        regressand = nan;     % if you wanna do a regression
        sourceDimension       % originalDimension
    end
    
    properties (GetAccess = private)
        method;               % feature selection / dim. reduction method
    end
    
    properties (SetAccess = public)
        description = 'unkown';% Human readable Description, e.g. a pattern name b01, b02, ..
        prediction             % dunno if needed ?
    end
    properties (Abstract)
        ImgType;               % Image Type, like con, beta, timeseries....
    end
    
    % Abstract method, which inits the pattern. After succesfull loading,
    % Prop. isLoaded should be true!
    methods (Abstract)
        % loads data
        this = generatePattern(this)
        % remove data and free memory
        this = freePattern(this)
        % return markers, that are unique for each subject
        markers = getUniqueMarkers(this,markers)
    end
    
    methods
        
        % return pattern as Vector
        function vector = get(this)
            vector = reshape(this.VoxelValues, getVoxelCount(this), 1);
        end
        
        % returns the Number of Voxels (...which are loaded!)
        function vc = getVoxelCount(this)
            % if loaded return number of voxels, else return zero
            if this.isLoaded
                vc = numel(this.VoxelValues);
            else
                vc = 0;
            end
        end
        
        % set Feature Selection Method
        function this = setFeatureSelection(this,method)
            if isa(method,'mania_fs')
                this.method = method;
            else
                throw(MException('mania:WrongFsMethodClass','Seems not to be a FS class!'));
            end
        end
        
        function this = setRegressand(this,reg)
            % set regressand (for regeression methods)
            this.regressand = reg;
        end
        
        function reg = getRegressand(this)
            % get regressand (for regression methods)
            reg = this.regressand;
        end
        
        function fs_method = getFeatureSelection(this)
            % get pattern preparation Method
            if ~isempty(this.method)
                fs_method = this.method;
            else
                throw(MException('mania:NullMethod','dimensionaltiy reduction method was not set!'));
            end
        end
        
        function show(this)
            % display a human readable summary of the pattern object
            fprintf('\nPattern \n');
            fprintf('   Description:     %s\n',this.description);
            fprintf('   Image Type:      %s\n',this.ImgType);
            fprintf('   Loaded:          %s\n',this.isLoadedAsString);
            if this.isLoaded
                fprintf('   Voxels:          %d\n',this.getVoxelCount);
            end
        end
        
        function new = copy(this)
            % returns copy of the pattern object
            new = feval(class(this));
            new.sourceDimension = this.sourceDimension;
            new.VoxelValues = this.VoxelValues;
            new.isLoaded = this.isLoaded;
            new.regressand = this.regressand;
            new.description = this.description;
            new.prediction = this.prediction;
            new.ImgType = this.ImgType;
            new.method = this.method;
        end
        
        
    end
    
    % Returns isLoaded as String
    methods (Access = private)
        function loaded = isLoadedAsString(jp)
            strs = {'Not yet';'Yes'};
            loaded = strs{jp.isLoaded+1};
        end
    end
end