% Pattern Parser (Abstract)
%   handles the parsing of patterns
%   cares, that patterns are in the right format, that are expected by the
%   algorithms
classdef mania_patternParser < handle
    
    properties (SetAccess = protected)
        classCollection;                    % classCollection, i.e. 2 or more classes with patterns
        preprocessors = {};                 % feature Selection methods, that apply BEFORE crossvalidation
        intraValidationProcessors = {};     % feature Selection methods, that apply DURING crossvalidation
        patternMat = [];                    % PatternMatrix
        patternLabels;                      % Labels of the patterns
        serviceMethods;                     % mania_patternServiceMethods
    end
    
    methods (Abstract)
        init(pp);                           % applys feature selection, formatting of the patternMat
        [patternMat labels] = get(pp,trainIndice,testIndice); %returns pattern Matrix of training Patterns
    end
    
    methods
        
        function this = mania_patternParser(classCollection)
            % Constructor
            %       only needs the classCollection
            if ~isa(classCollection,'mania_classCollection')
                throw(MException('mania:WrongClass','This is not a classCollection-Class!')); 
            else
                this.classCollection = classCollection;
                this.serviceMethods  = mania_patternServiceMethods(this.classCollection);
            end
        end
        
        function setClassCollection(this,classCollection)
            % set / change the classCollection
            if isa(classCollection,'mania_classCollection')
                this.classCollection = classCollection;
            else
                throw(MException('mania:WrongClass','This is not a classCollection-Class!')); 
            end
        end
        
        function val = size(this)
            % return the size of the pattern
           val = size(this.patternMat,2); 
        end
        
        function this = addPreprocessors(this,processor)
            % add feature Selection methods, that apply BEFORE crossvalidation
            if isa(processor,'mania_preprocessor')
                processor.setServiceMethods(this.serviceMethods);
                this.preprocessors{size(this.preprocessors,2)+1} = processor;
            else
                throw(MException('mania:WrongClass','This is not a Preprocessor-Class!'));
            end
        end
        
        function preprocessors = getPreprocessors(this,idx)
            % return feature Selection methods, that apply BEFORE crossvalidation
            preprocessors = this.preprocessors{idx};
        end
        
        function pp = removePreprocessor(pp,idx)
            % remove preparation method with index idx
            pp.preprocessors(idx) = [];
        end
        
        function this = addIntraValidationProcessors(this,processor)
            % add feature Selection methods, that apply DURING crossvalidation
            if isa(processor,'mania_processor')
                processor.setServiceMethods(this.serviceMethods);
                this.intraValidationProcessors{size(this.intraValidationProcessors,2)+1} = processor;
            else
                throw(MException('mania:WrongClass','This is not a Preprocessor-Class!'));
            end
        end
        
        function preprocessors = getIntraValidationProcessors(this,idx)
            % return feature Selection methods, that apply DURING crossvalidation
            preprocessors = this.IntraValidationProcessors{idx};
        end
        
        function pp = removeIntraValidationProcessor(pp,idx)
            % remove embedded method with index idx
            pp.IntraValidationProcessors(idx) = [];
        end        
        
        function new = copy(this)
            % return copy of this object
            new = feval(class(this),this.classCollection);
            new.classCollection = this.classCollection;
            new.preprocessors = this.preprocessors;
            new.intraValidationProcessors = this.intraValidationProcessors;
            new.patternMat = this.patternMat;
            new.patternLabels = this.patternLabels;
        end
        
    end

    methods (Access = protected)
        % applies feature Selection methods, that apply BEFORE crossvalidation
        function pattern = applyProcessors(this,pattern)
           for k=1:numel(this.preprocessors)
              pattern = this.preprocessors{k}.operate(pattern);
           end
        end
    end
end