% Just applies a mask to all the subjects
% If there are more than one image, the mask is applied to ALL images.
classdef mania_pre_roi < mania_preprocessor
    
    properties (SetAccess = protected)
        mask;                       % mask that schould be applied
        fname;                      % if a file was specified
    end
    
    properties (SetAccess = public)
        maskName;                   % name of mask
    end
    
    methods (Static)
        function desc = description
            desc = 'Region of Interest';
        end
        function shortName = name
            shortName = 'Region of interest';
        end
        function    [state, msg] = checkSetup
            if exist('spm','file')
                state = true;
                msg = 'SPM to read image files: ✓';
            else
                state = false;
                msg = 'No SPM version found (for reading image files).';
            end
        end
    end
    
    
    methods
        function bool = isConfigurable(preprop)
            bool = true;
        end
        
        function this =  uiconfigure(this)
            [filename, pathname, filterindex] = uigetfile( '*.img;*.nii', 'Select a mask');
            if ischar(filename)
                this.setMask([pathname filename]);
                this.maskName = [pathname filename];
            end
        end
        
        function patternOut = operate(this, patternIn, trainLabels)
            % Operate will apply the specified mask
            if isempty(this.mask)
                throw(MException('mania:MaskWasNotSet','Mask (Region of interest) was not set'));
            end
            mask = this.mask;
            origDim = this.serviceMethods.requestOrigDimension;
            % if dimension is unkown: trust user (that hes is aware of things he is doing), but at least throw a warning!
            if isempty(origDim)
                warning('Could not determine dimension of source images. - Proceed anyway.'); 
            % if dimension is known and differs from images: try to resize
            % with SPM's reslicing
            elseif numel(origDim)==1 || isequal(origDim{:})
                if origDim{1} ~= size(mask)
                   try
                        disp('Dimension missmatch of ROI-mask. Will try to resize...');
                        newMask = this.serviceMethods.requestImageResize(this.fname);
                        this.setMask(newMask);
                        patternOut = operate(this,patternIn);
                        return
                   catch exp
                       warning('Could not resize mask! Please try to resize manually.')
                       rethrow(exp);
                   end
                end
            else
                warning('Dimension of images differ. - Proceed anyway.'); 
            end
            if ~isempty(this.previousOperations)
                for k =1:numel(this.previousOperations)
                    mask = this.previousOperations{k}.reoperate(mask);
                end
            end
            try
                patternOut = patternIn(logical(reshape(mask,numel(mask),1)),:);
            catch
                error('Could not reshape mask to a vector - Something is messed up with image dimensions!');
            end
        end
        
        function patternOut = reoperate(this,patternIn)
            patternIn = reshape(patternIn,numel(patternIn),1);
            mask = this.mask;
            if ~isempty(this.previousOperations)
                for k =1:numel(this.previousOperations)
                    mask = this.previousOperations{k}.reoperate(mask);
                end
            end
            if size(patternIn,1) < size(mask,1) && ~mod(size(mask,1),size(patternIn,1))
                for k=1:size(mask,1)/size(patternIn,1)
                    patternIn = [patternIn; patternIn];
                end
            end
            mask = logical(mask);
            patternOut = patternIn(mask,:);
        end
        
        function patternOut = reconstruct(this, patternIn)
            % Reconstructs oringinal mask...
            % Previously masked voxels are set to zero (no influence on accuracy)
            mask = this.mask;
            % repeat previously actions, like mask NaNs on the ROI.
            if ~isempty(this.previousOperations)
                for k =1:numel(this.previousOperations)
                    mask = this.previousOperations{k}.reoperate(mask);
                end
            end
            patternOut = zeros(numel(mask),size(patternIn,1));
            patternOut(logical(mask),:) = patternIn;
        end
        
        function this = setMask(this, roi)
            % set mask
            %   The roi can be a path to a mask in SPM-Format or
            %   a numeric matrix, that represents the mask.
            fname = 'unknown';
            if ischar(roi)              % if its char -> its a filename
                fname = roi;
                try
                    roi = spm_read_vols(spm_vol(roi));
                    this.fname = fname;
                catch
                    throw(MException('mania:CouldNotReadImage','The mask could not be loaded'));
                end
            end
            if isnumeric(roi) || islogical(roi)  % if its numeric -> its the ROI itself!
                if numel(roi(roi~=0))>0
                    this.mask = roi;
                else
                    throw(MException('mania:CouldNotUseROI',['The mask ''' this.maskName ''' (' fname ') index contains no voxels!']));
                end
            else
                throw(MException('mania:WrongMaskFormat','The mask should contain logical values'));
            end
        end
        
        function new = copy(this)
            new = copy@mania_preprocessor(this);
            new.mask = this.mask;
            new.maskName = this.maskName;
            new.fname = this.fname;
        end
        
        function viewSettings(this)
            viewSettings@mania_preprocessor(this);
            fprintf('  ROI: %s\n',this.maskName);
            fprintf('\tVoxels included: %i',sum(reshape(this.mask,numel(this.mask),1)));
        end
        
        
    end
    
end